# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import warnings

from django import VERSION as DJANGO_VERSION
if DJANGO_VERSION < (2, 0):
    from django.conf.urls import include, url
else:    
    from django.urls import include, path
    
from shop.modifiers.pool import cart_modifiers_pool


urlpatterns = []

# For every payment modifier, load the URLs from the associated `payment_provider`.
for modifier in cart_modifiers_pool.get_payment_modifiers():
    try:
        namespace = modifier.payment_provider.namespace
        if DJANGO_VERSION < (2, 0):
            regexp = r'^{}/'.format(namespace)
            urls = modifier.payment_provider.get_urls()
            provider_url = url(regexp, include(urls, namespace=namespace))
            if provider_url.describe() not in [u.describe() for u in urlpatterns]:
                urlpatterns.append(provider_url)
        else:
            path_ = '{}/'.format(namespace)
            urls = modifier.payment_provider.get_urls()
            provider_url = path( path_, include((urls, 'url_payment'), namespace=namespace))
            if provider_url.url_patterns != []:
                urlpatterns.append(provider_url)
    except AttributeError as err:
        warnings.warn(err.message)



from django.conf import settings
from django.apps import apps
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist, ValidationError
from django.forms import fields, widgets, ModelChoiceField
from django.template import TemplateDoesNotExist
from django.template.loader import select_template
from django.utils.html import format_html
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _, pgettext_lazy
from entangled.forms import EntangledModelFormMixin, get_related_object

if 'cmsplugin_cascade' not in settings.INSTALLED_APPS:
    raise ImproperlyConfigured("Please add 'cmsplugin_cascade' to your INSTALLED_APPS")

from cms.plugin_pool import plugin_pool
from cmsplugin_cascade.plugin_base import CascadePluginBase
from cmsplugin_cascade.link.forms import LinkForm
from cmsplugin_cascade.link.plugin_base import LinkPluginBase
from django_select2.forms import HeavySelect2Widget
from shop.conf import app_settings
from shop.forms.base import DialogFormMixin
from shop.models.cart import CartModel
from shop.models.product import ProductModel


class ShopPluginBase(CascadePluginBase):
    module = "Shop"
    require_parent = False
    allow_children = False


class ProductSelectField(ModelChoiceField):
    widget = HeavySelect2Widget(data_view='shop:select-product')

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('queryset', ProductModel.objects.all())
        super().__init__(*args, **kwargs)


class CatalogLinkForm(LinkForm):
    """
    Alternative implementation of `cmsplugin_cascade.link.forms.LinkForm`, which allows to link onto
    the Product model, using its method ``get_absolute_url``.

    Note: In this form class the field ``product`` is missing. It is added later, when the shop's
    Product knows about its materialized model.
    """
    LINK_TYPE_CHOICES = [
        ('cmspage', _("CMS Page")),
        ('product', _("Product")),
        ('download', _("Download File")),
        ('exturl', _("External URL")),
        ('email', _("Mail To")),
    ]

    product = ProductSelectField(
        label=_("Product"),
        required=False,
        help_text=_("An internal link onto a product from the catalog"),
    )

    class Meta:
        entangled_fields = {'glossary': ['product']}

    def clean(self):
        cleaned_data = super().clean()
        link_type = cleaned_data.get('link_type')
        error = None
        if link_type == 'product':
            if cleaned_data['product'] is None:
                error = ValidationError(_("Product to link to is missing."))
                self.add_error('product', error)
        if error:
            raise error
        return cleaned_data


class CatalogLinkPluginBase(LinkPluginBase):
    """
    Alternative implementation to ``cmsplugin_cascade.link.DefaultLinkPluginBase`` which adds
    another link type, namely "Product", to set links onto arbitrary products of this shop.
    """
    ring_plugin = 'ShopLinkPlugin'

    class Media:
        js = ['admin/js/jquery.init.js', 'shop/js/admin/shoplinkplugin.js']

    @classmethod
    def get_link(cls, obj):
        link_type = obj.glossary.get('link_type')
        if link_type == 'product':
            relobj = get_related_object(obj.glossary, 'product')
            if relobj:
                return relobj.get_absolute_url()
        else:
            return super().get_link(obj) or link_type


class DialogPluginBaseForm(EntangledModelFormMixin):
    RENDER_CHOICES = [
        ('form', _("Form dialog")),
        ('summary', _("Static summary")),
    ]

    render_type = fields.ChoiceField(
        choices=RENDER_CHOICES,
        widget=widgets.RadioSelect,
        label=_("Render as"),
        initial='form',
        help_text=_("A dialog can also be rendered as a box containing a read-only summary."),
    )

    headline_legend = fields.BooleanField(
        label=_("Headline Legend"),
        initial=True,
        required=False,
        help_text=_("Render a legend inside the dialog's headline."),
    )

    class Meta:
        entangled_fields = {'glossary': ['render_type', 'headline_legend']}


class DialogFormPluginBase(ShopPluginBase):
    """
    Base class for all plugins adding a dialog form to a placeholder field.
    """
    require_parent = True
    parent_classes = ['BootstrapColumnPlugin', 'ProcessStepPlugin', 'BootstrapPanelPlugin',
        'SegmentPlugin', 'SimpleWrapperPlugin', 'ValidateSetOfFormsPlugin']
    form = DialogPluginBaseForm

    @classmethod
    def register_plugin(cls, plugin):
        """
        Register plugins derived from this class with this function instead of
        `plugin_pool.register_plugin`, so that dialog plugins without a corresponding
        form class are not registered.
        """
        if not issubclass(plugin, cls):
            msg = "Can not register plugin class `{}`, since is does not inherit from `{}`."
            raise ImproperlyConfigured(msg.format(plugin.__name__, cls.__name__))
        plugin_pool.register_plugin(plugin)

    def get_form_class(self, instance):
        try:
            return import_string(self.form_class)
        except AttributeError:
            msg = "Can not register plugin class '{}', since it neither defines 'form_class' " \
                  "nor overrides 'get_form_class()'."
            raise ImproperlyConfigured(msg.format(self.__name__))

    @classmethod
    def get_identifier(cls, instance):
        render_type = instance.glossary.get('render_type')
        render_type = dict(cls.form.RENDER_CHOICES).get(render_type, '')
        return format_html(pgettext_lazy('get_identifier', "as {}"), render_type)

    def get_form_data(self, context, instance, placeholder):
        """
        Returns data to initialize the corresponding dialog form.
        This method must return a dictionary containing
         * either `instance` - a Python object to initialize the form class for this plugin,
         * or `initial` - a dictionary containing initial form data, or if both are set, values
           from `initial` override those of `instance`.
        """
        if issubclass(self.get_form_class(instance), DialogFormMixin):
            try:
                cart = CartModel.objects.get_from_request(context['request'])
                cart.update(context['request'])
            except CartModel.DoesNotExist:
                cart = None
            return {'cart': cart}
        return {}

    def get_render_template(self, context, instance, placeholder):
        render_type = instance.glossary.get('render_type')
        if render_type not in ('form', 'summary',):
            render_type = 'form'
        try:
            template_names = [
                '{0}/checkout/{1}'.format(app_settings.APP_LABEL, self.template_leaf_name).format(render_type),
                'shop/checkout/{}'.format(self.template_leaf_name).format(render_type),
            ]
            return select_template(template_names)
        except (AttributeError, TemplateDoesNotExist):
            return self.render_template

    def render(self, context, instance, placeholder):
        """
        Return the context to render a DialogFormPlugin
        """
        request = context['request']
        form_data = self.get_form_data(context, instance, placeholder)
        request._plugin_order = getattr(request, '_plugin_order', 0) + 1
        if not isinstance(form_data.get('initial'), dict):
            form_data['initial'] = {}
        form_data['initial'].update(plugin_id=instance.pk, plugin_order=request._plugin_order)
        bound_form = self.get_form_class(instance)(**form_data)
        context[bound_form.form_name] = bound_form
        context['headline_legend'] = bool(instance.glossary.get('headline_legend', True))
        return self.super(DialogFormPluginBase, self).render(context, instance, placeholder)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class ProductNotAvailable(Exception):
    """
    The product being purchased isn't available anymore.
    """
    def __init__(self, product):
        self.product = product
        msg = "Product {} isn't available anymore."
        super(ProductNotAvailable, self).__init__(msg.format(product.product_code))



from django.template.loader import select_template
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from django.utils.module_loading import import_string
from entangled.forms import EntangledModelFormMixin
from cms.plugin_pool import plugin_pool
from cmsplugin_cascade.link.cms_plugins import LinkElementMixin, LinkPluginBase
from cmsplugin_cascade.link.forms import LinkForm
from djng.forms.fields import ChoiceField
from shop.conf import app_settings

AUTH_FORM_TYPES = [
    ('login', _("Login Form")),
    ('login-reset-request', _("Login & Reset Request"), 'login'),
    ('logout', _("Logout Form")),
    ('login-logout', _("Shared Login/Logout Form"), 'login'),
    ('password-reset-request', _("Request Password Reset")),
    ('password-reset-confirm', _("Confirm Password Reset")),
    ('password-change', _("Change Password Form")),
    ('register-user', _("Register User"), 'shop.forms.auth.RegisterUserForm'),
    ('continue-as-guest', _("Continue as guest")),
]


class ShopAuthFormMixin(EntangledModelFormMixin):
    form_type = ChoiceField(
        label=_("Rendered Form"),
        choices=[ft[:2] for ft in AUTH_FORM_TYPES],
        help_text=_("Select the appropriate form for various authentication purposes."),
    )

    class Meta:
        entangled_fields = {'glossary': ['form_type']}


class ShopAuthForm(LinkForm, ShopAuthFormMixin):
    LINK_TYPE_CHOICES = [
        ('RELOAD_PAGE', _("Reload Page")),
        ('cmspage', _("CMS Page")),
        ('DO_NOTHING', _("Do Nothing")),
    ]


class ShopAuthenticationPlugin(LinkPluginBase):
    """
    A placeholder plugin which provides various authentication forms, such as login-, logout-,
    register-, and other forms. They can be added any placeholder using the Cascade framework.
    """
    name = _("Authentication Forms")
    module = "Shop"
    parent_classes = ['BootstrapColumnPlugin']
    model_mixins = (LinkElementMixin,)
    form = ShopAuthForm
    cache = False

    @classmethod
    def get_identifier(cls, instance):
        identifier = super(ShopAuthenticationPlugin, cls).get_identifier(instance)
        content = dict(ft[:2] for ft in AUTH_FORM_TYPES).get(instance.glossary.get('form_type'), _("unknown"))
        return format_html('{0}{1}', identifier, content)

    def get_render_template(self, context, instance, placeholder):
        form_type = instance.glossary.get('form_type')
        template_names = [
            '{}/auth/{}.html'.format(app_settings.APP_LABEL, form_type),
            'shop/auth/{}.html'.format(form_type),
            'shop/auth/form-not-found.html',
        ]
        return select_template(template_names)

    def render(self, context, instance, placeholder):
        """
        Return the context to render a DialogFormPlugin
        """
        form_type = instance.glossary.get('form_type')
        if form_type:
            try:
                # prevent a malicious database entry to import an ineligible file
                form_type = AUTH_FORM_TYPES[[ft[0] for ft in AUTH_FORM_TYPES].index(form_type)]
                FormClass = import_string(form_type[2])
            except ValueError:
                context['form_name'] = 'not_found_form'
            except IndexError:
                form_name = form_type[0].replace('-', '_')
                context['form_name'] = '{0}_form'.format(form_name)
            except ImportError:
                form_name = form_type[2]
                context['form_name'] = '{0}_form'.format(form_name)
            else:
                context['form_name'] = FormClass.form_name
                context[FormClass.form_name] = FormClass()
        context['proceed_with'] = instance.link
        return self.super(ShopAuthenticationPlugin, self).render(context, instance, placeholder)

plugin_pool.register_plugin(ShopAuthenticationPlugin)



# -*- coding: utf-8 -*-
"""
In django-SHOP, a Commodity product-model is considered a very basic product without any attributes,
which can be used on a generic CMS page to describe anything.
"""
from __future__ import unicode_literals

from django.conf import settings
from django.core.validators import MinValueValidator
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from cms.models.fields import PlaceholderField
from filer.fields import image
from djangocms_text_ckeditor.fields import HTMLField
from polymorphic.query import PolymorphicQuerySet
from shop.conf import app_settings
from shop.models.product import BaseProduct, BaseProductManager, CMSPageReferenceMixin, AvailableProductMixin
from shop.models.defaults.mapping import ProductPage
from shop.money.fields import MoneyField


class CommodityMixin(AvailableProductMixin):
    """
    Common methods used by both default Commodity models.
    """
    def get_price(self, request):
        return self.unit_price


if settings.USE_I18N:
    assert 'parler' in settings.INSTALLED_APPS, "Requires `django-parler`, if configured as multilingual project"
    from parler.managers import TranslatableManager, TranslatableQuerySet
    from parler.models import TranslatableModelMixin, TranslatedFieldsModel
    from parler.fields import TranslatedField


    class ProductQuerySet(TranslatableQuerySet, PolymorphicQuerySet):
        pass


    class ProductManager(BaseProductManager, TranslatableManager):
        queryset_class = ProductQuerySet


    @python_2_unicode_compatible
    class Commodity(CMSPageReferenceMixin, TranslatableModelMixin, CommodityMixin, BaseProduct):
        """
        Generic Product Commodity to be used whenever the merchant does not require product specific
        attributes and just required a placeholder field to add arbitrary data.
        """
        # common product fields
        product_code = models.CharField(
            _("Product code"),
            max_length=255,
            unique=True,
        )

        unit_price = MoneyField(
            _("Unit price"),
            decimal_places=3,
            help_text=_("Net price for this product"),
        )

        # controlling the catalog
        order = models.PositiveIntegerField(
            verbose_name=_("Sort by"),
            db_index=True,
        )

        cms_pages = models.ManyToManyField(
            'cms.Page',
            through=ProductPage,
            help_text=_("Choose list view this product shall appear on."),
        )

        sample_image = image.FilerImageField(
            verbose_name=_("Sample Image"),
            blank=True,
            null=True,
            default=None,
            on_delete=models.SET_DEFAULT,
            help_text=_("Sample image used in the catalog's list view."),
        )

        show_breadcrumb = models.BooleanField(
            _("Show Breadcrumb"),
            default=True,
            help_text=_("Shall the detail page show the product's breadcrumb."),
        )

        placeholder = PlaceholderField("Commodity Details")

        quantity = models.PositiveIntegerField(
            _("Quantity"),
            default=0,
            validators=[MinValueValidator(0)],
            help_text=_("Available quantity in stock")
        )

        # translatable fields for the catalog's list- and detail views
        product_name = TranslatedField()
        slug = TranslatedField()
        caption = TranslatedField()

        # filter expression used to search for a product item using the Select2 widget
        lookup_fields = ['product_code__startswith', 'product_name__icontains']

        objects = ProductManager()

        class Meta:
            app_label = app_settings.APP_LABEL
            ordering = ['order']
            verbose_name = _("Commodity")
            verbose_name_plural = _("Commodities")

        def __str__(self):
            return self.product_code


    class CommodityTranslation(TranslatedFieldsModel):
        master = models.ForeignKey(
            Commodity,
            related_name='translations',
            on_delete=models.CASCADE,
            null=True,
        )

        product_name = models.CharField(
            max_length=255,
            verbose_name=_("Product Name"),
        )

        slug = models.SlugField(verbose_name=_("Slug"))

        caption = HTMLField(
            verbose_name=_("Caption"),
            blank=True,
            null=True,
            help_text=_("Short description for the catalog list view."),
        )

        class Meta:
            app_label = app_settings.APP_LABEL
            unique_together = [('language_code', 'master')]

else:

    @python_2_unicode_compatible
    class Commodity(CMSPageReferenceMixin, CommodityMixin, BaseProduct):
        """
        Generic Product Commodity to be used whenever the merchant does not require product specific
        attributes and just required a placeholder field to add arbitrary data.
        """
        # common product fields
        product_name = models.CharField(
            max_length=255,
            verbose_name=_("Product Name"),
        )

        product_code = models.CharField(
            _("Product code"),
            max_length=255,
            unique=True,
        )

        unit_price = MoneyField(
            _("Unit price"),
            decimal_places=3,
            help_text=_("Net price for this product"),
        )

        # controlling the catalog
        order = models.PositiveIntegerField(
            verbose_name=_("Sort by"),
            db_index=True,
        )

        cms_pages = models.ManyToManyField(
            'cms.Page',
            through=ProductPage,
            help_text=_("Choose list view this product shall appear on."),
        )

        sample_image = image.FilerImageField(
            verbose_name=_("Sample Image"),
            blank=True,
            null=True,
            default=None,
            on_delete=models.SET_DEFAULT,
            help_text=_("Sample image used in the catalog's list view."),
        )

        show_breadcrumb = models.BooleanField(
            _("Show Breadcrumb"),
            default=True,
            help_text=_("Shall the detail page show the product's breadcrumb."),
        )

        placeholder = PlaceholderField("Commodity Details")

        quantity = models.PositiveIntegerField(
            _("Quantity"),
            default=0,
            validators=[MinValueValidator(0)],
            help_text=_("Available quantity in stock")
        )

        # common fields for the catalog's list- and detail views
        slug = models.SlugField(verbose_name=_("Slug"))

        caption = HTMLField(
            verbose_name=_("Caption"),
            blank=True,
            null=True,
            help_text=_("Short description for the catalog list view."),
        )

        # filter expression used to search for a product item using the Select2 widget
        lookup_fields = ['product_code__startswith', 'product_name__icontains']

        objects = BaseProductManager()

        class Meta:
            app_label = app_settings.APP_LABEL
            ordering = ('order',)
            verbose_name = _("Commodity")
            verbose_name_plural = _("Commodities")

        def __str__(self):
            return self.product_code



# Generated by Django 2.2.9 on 2019-12-24 07:27

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0009_delete_email'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='mail_template',
            field=models.ForeignKey(limit_choices_to=models.Q(('language__isnull', True), ('language', ''), _connector='OR'), on_delete=django.db.models.deletion.CASCADE, to='post_office.EmailTemplate', verbose_name='Template'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='recipient',
            field=models.ForeignKey(limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Recipient'),
        ),
        migrations.AlterField(
            model_name='notificationattachment',
            name='attachment',
            field=filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='email_attachment', to='filer.File'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from shop.modifiers.base import BaseCartModifier


class ShippingModifier(BaseCartModifier):
    """
    Base class for all shipping modifiers. The purpose of a shipping modifier is to calculate the shipping costs and/or
    prevent its usage, in case products in the cart can not be shipped to the desired destination. The merchant may
    either append a single shipping modifier to the list of ``SHOP_CART_MODIFIERS``, or create a sublist of shipping
    modifier and append this sublist to ``SHOP_CART_MODIFIERS``. The latter is useful to instantiate the same shipping
    modifier multiple times for different shipping carriers using the same interface.

    The merchant must specify at least one shipping modifier. If there is more than one, the merchant shall offer a
    select option during checkout. In django-SHOP, one can use the plugin **Shipping Method Form** to render such a
    select option.

    Each shipping modifier can add a surcharge on the current cart. If weight affects the shipping price, it shall be
    summed up inside the method `add_extra_cart_row` and used to lookup the shipping costs.
    """
    def get_choice(self):
        """
        :returns: A tuple consisting of 'value, label' used by the shipping form dialog to render
        the available shipping choices.
        """
        raise NotImplemented("{} must implement method `get_choice()`.".format(self.__class__))

    def is_active(self, shipping_modifier):
        """
        :returns: ``True`` if this shipping modifier is the actively selected one.
        """
        return shipping_modifier == self.identifier

    def is_disabled(self, cart):
        """
        Hook method to be overridden by the concrete shipping modifier. Shall be used to
        temporarily disable a shipping method, in case the cart does not fulfill certain criteria,
        for instance an undeliverable destination address.

        :returns: ``True`` if this shipping modifier is disabled for the current cart.
        """
        return False

    def update_render_context(self, context):
        """
        Hook to update the rendering context with shipping specific data.
        """
        from shop.models.cart import CartModel

        if 'shipping_modifiers' not in context:
            context['shipping_modifiers'] = {}
        try:
            cart = CartModel.objects.get_from_request(context['request'])
            if self.is_active(cart.extra.get('shipping_modifier')):
                cart.update(context['request'])
                data = cart.extra_rows[self.identifier].data
                data.update(modifier=self.identifier)
                context['shipping_modifiers']['initial_row'] = data
        except (KeyError, CartModel.DoesNotExist):
            pass

    def ship_the_goods(self, delivery):
        """
        Hook to be overridden by the active shipping modifier. It should be used to perform the
        shipping request.
        """
        delivery.shipped_at = timezone.now()


class SelfCollectionModifier(ShippingModifier):
    """
    This modifiers has not influence on the cart final. It can be used,
    to enable the customer to pick up the products in the shop.
    """
    identifier = 'self-collection'

    def get_choice(self):
        return (self.identifier, _("Self-collection"))

    def ship_the_goods(self, delivery):
        if not delivery.shipping_id:
            delivery.shipping_id = str(delivery.id)
        super(SelfCollectionModifier, self).ship_the_goods(delivery)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from shop.modifiers.base import BaseCartModifier
from shop.payment.providers import PaymentProvider, ForwardFundPayment


class PaymentModifier(BaseCartModifier):
    """
    Base class for all payment modifiers. The purpose of a payment modifier is to calculate the payment surcharge and/or
    prevent its usage, in case the choosen payment method is not available for the given customer. The merchant may
    either append a single payment modifier to the list of ``SHOP_CART_MODIFIERS``, or create a sublist of payment
    modifier and append this sublist to ``SHOP_CART_MODIFIERS``. The latter is useful to instantiate the same payment
    modifier multiple times for different payment service providers using the same interface.

    The merchant must specify at least one payment modifier. If there is more than one, the merchant shall offer a
    select option during checkout. In django-SHOP, one can use the plugin **Payment Method Form** to render such a
    select option.

    Each payment modifier can add a surcharge on the current cart.
    """
    def __init__(self):
        assert isinstance(getattr(self, 'payment_provider', None), PaymentProvider), \
            "Each Payment modifier class requires a Payment Provider"
        super(PaymentModifier, self).__init__()

    @property
    def identifier(self):
        """
        Default identifier for payment providers.
        """
        return self.payment_provider.namespace

    def get_choice(self):
        """
        :returns: A tuple consisting of 'value, label' used by the payment form dialog to render
        the available payment choices.
        """
        raise NotImplemented("{} must implement method `get_choice()`.".format(self.__class__))

    def is_active(self, payment_modifier):
        """
        :returns: ``True`` if this payment modifier is active.
        """
        assert hasattr(self, 'payment_provider'), "A Payment Modifier requires a Payment Provider"
        return payment_modifier == self.identifier

    def is_disabled(self, cart):
        """
        Hook method to be overridden by the concrete payment modifier. Shall be used to
        temporarily disable a payment method, in case the cart does not fulfill certain criteria,
        for instance a too small total.

        :returns: ``True`` if this payment modifier is disabled for the current cart.
        """
        return False

    def update_render_context(self, context):
        """
        Hook to update the rendering context with payment specific data.
        """
        from shop.models.cart import CartModel

        if 'payment_modifiers' not in context:
            context['payment_modifiers'] = {}
        try:
            cart = CartModel.objects.get_from_request(context['request'])
            if self.is_active(cart.extra.get('payment_modifier')):
                cart.update(context['request'])
                data = cart.extra_rows[self.identifier].data
                data.update(modifier=self.identifier)
                context['payment_modifiers']['initial_row'] = data
        except (KeyError, CartModel.DoesNotExist):
            pass


class PayInAdvanceModifier(PaymentModifier):
    """
    This modifiers has no influence on the cart final. It can be used,
    to enable the customer to pay the products on delivery.
    """
    payment_provider = ForwardFundPayment()

    def get_choice(self):
        return (self.payment_provider.namespace, _("Pay in advance"))



from django.forms import fields, widgets
from django.template import engines, TemplateDoesNotExist
from django.template.loader import select_template, get_template
from django.utils.translation import ugettext_lazy as _
from django.utils.html import mark_safe
from entangled.forms import EntangledModelFormMixin
from cms.plugin_pool import plugin_pool
from cmsplugin_cascade.plugin_base import TransparentWrapper
from shop.cascade.extensions import ShopExtendableMixin, LeftRightExtensionMixin
from shop.cascade.plugin_base import ShopPluginBase
from shop.conf import app_settings
from shop.models.cart import CartModel
from shop.serializers.cart import CartSerializer


class ShopCartPluginForm(EntangledModelFormMixin):
    CHOICES = [
        ('editable', _("Editable Cart")),
        ('static', _("Static Cart")),
        ('summary', _("Cart Summary")),
        ('watch', _("Watch List")),
    ]

    render_type = fields.ChoiceField(
        choices=CHOICES,
        widget=widgets.RadioSelect,
        label=_("Render as"),
        initial='editable',
        help_text=_("Shall the cart be editable or a static summary?"),
    )

    class Meta:
        entangled_fields = {'glossary': ['render_type']}


class ShopCartPlugin(LeftRightExtensionMixin, TransparentWrapper, ShopPluginBase):
    name = _("Shopping Cart")
    require_parent = True
    parent_classes = ['BootstrapColumnPlugin']
    cache = False
    allow_children = True
    form = ShopCartPluginForm
    model_mixins = (ShopExtendableMixin,)

    @classmethod
    def get_identifier(cls, instance):
        render_type = instance.glossary.get('render_type')
        return mark_safe(dict(cls.form.CHOICES).get(render_type, ''))

    def get_render_template(self, context, instance, placeholder):
        render_template = instance.glossary.get('render_template')
        if render_template:
            return get_template(render_template)
        render_type = instance.glossary.get('render_type')
        try:
            return select_template([
                '{}/cart/{}.html'.format(app_settings.APP_LABEL, render_type),
                'shop/cart/{}.html'.format(render_type),
            ])
        except TemplateDoesNotExist:
            return get_template('shop/cart/editable.html')

    def render(self, context, instance, placeholder):
        try:
            cart = CartModel.objects.get_from_request(context['request'])
            context['is_cart_filled'] = cart.items.exists()
            render_type = instance.glossary['render_type']
            if render_type == 'static':
                # update context for static cart with items to be endered as HTML
                cart_serializer = CartSerializer(cart, context=context, label='cart', with_items=True)
                context['cart'] = cart_serializer.data
            elif render_type == 'summary':
                # update context for cart summary to be endered as HTML
                cart_serializer = CartSerializer(cart, context=context, label='cart')
                context['cart'] = cart_serializer.data
        except (KeyError, CartModel.DoesNotExist):
            pass
        return self.super(ShopCartPlugin, self).render(context, instance, placeholder)

plugin_pool.register_plugin(ShopCartPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import url
from django.contrib import admin
from django.db.models.fields import Field, FieldDoesNotExist
from django.forms import widgets
from django.http import HttpResponse
from django.template.loader import select_template
from django.urls import reverse, NoReverseMatch
from django.utils.html import format_html
from django.utils.translation import pgettext_lazy

from fsm_admin.mixins import FSMTransitionMixin

from shop.conf import app_settings
from shop.models.customer import CustomerModel
from shop.models.order import OrderItemModel, OrderPayment
from shop.modifiers.pool import cart_modifiers_pool
from shop.serializers.order import OrderDetailSerializer
from shop.transition import transition_change_notification


class OrderPaymentInline(admin.TabularInline):
    model = OrderPayment
    extra = 0

    def get_formset(self, request, obj=None, **kwargs):
        """
        Convert the field `payment_method` into a select box with all possible payment methods.
        """
        choices = [pm.get_choice() for pm in cart_modifiers_pool.get_payment_modifiers()]
        kwargs.update(widgets={'payment_method': widgets.Select(choices=choices)})
        formset = super(OrderPaymentInline, self).get_formset(request, obj, **kwargs)
        return formset

    def has_add_permission(self, request, obj=None):
        assert obj is not None, "An Order object can not be added through the Django-Admin"
        return obj.status in ['awaiting_payment', 'refund_payment']

    def has_delete_permission(self, request, obj=None):
        return False

    def get_max_num(self, request, obj=None, **kwargs):
        qs = self.model.objects.filter(order=obj)
        if self.has_add_permission(request, obj):
            return qs.count() + 1
        return qs.count()

    def get_fields(self, request, obj=None):
        if self.has_add_permission(request, obj):
            return ['amount', 'transaction_id', 'payment_method', 'created_at']
        return ['get_amount', 'transaction_id', 'payment_method', 'created_at']

    def get_readonly_fields(self, request, obj=None):
        if self.has_add_permission(request, obj):
            return ['created_at', 'get_amount']
        # unless we expect a payment, set all fields as readonly
        return self.get_fields(request, obj)

    def get_amount(self, obj):
        """Return amount using correct local format"""
        return obj.amount
    get_amount.short_description = pgettext_lazy('admin', "Amount Paid")


class OrderItemInline(admin.StackedInline):
    model = OrderItemModel
    extra = 0
    fields = [
        ('product_code', 'unit_price', 'line_total',),
        ('quantity',),
        'render_as_html_extra',
    ]
    readonly_fields = ['product_code', 'quantity', 'unit_price', 'line_total', 'render_as_html_extra']
    template = 'shop/admin/edit_inline/stacked-order.html'

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_max_num(self, request, obj=None, **kwargs):
        return self.model.objects.filter(order=obj).count()

    def render_as_html_extra(self, obj):
        item_extra_template = select_template([
            '{0}/admin/orderitem-{1}-extra.html'.format(app_settings.APP_LABEL, obj.product.product_model),
            '{0}/admin/orderitem-product-extra.html'.format(app_settings.APP_LABEL),
            'shop/admin/orderitem-product-extra.html',
        ])
        return item_extra_template.render(obj.extra)
    render_as_html_extra.short_description = pgettext_lazy('admin', "Extra data")


class StatusListFilter(admin.SimpleListFilter):
    title = pgettext_lazy('admin', "Status")
    parameter_name = 'status'

    def lookups(self, request, model_admin):
        lookups = dict(model_admin.model._transition_targets)
        lookups.pop('new')
        lookups.pop('created')
        return lookups.items()

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(status=self.value())
        return queryset


class BaseOrderAdmin(FSMTransitionMixin, admin.ModelAdmin):
    list_display = ['get_number', 'customer', 'status_name', 'get_total', 'created_at']
    list_filter = [StatusListFilter]
    fsm_field = ['status']
    date_hierarchy = 'created_at'
    inlines = [OrderItemInline]
    readonly_fields = ['get_number', 'status_name', 'get_total', 'get_subtotal',
                       'get_customer_link', 'get_outstanding_amount', 'created_at', 'updated_at',
                       'render_as_html_extra', 'stored_request', 'is_fully_paid']
    fields = ['get_number', 'status_name',
              ('created_at', 'updated_at'),
              'get_customer_link',
              ('get_subtotal', 'get_total', 'get_outstanding_amount', 'is_fully_paid'),
              'render_as_html_extra', 'stored_request']
    actions = None
    change_form_template = 'shop/admin/change_form.html'

    def __init__(self, *args, **kwargs):
        super(BaseOrderAdmin, self).__init__(*args, **kwargs)
        self.extra_template = select_template([
            '{}/admin/order-extra.html'.format(app_settings.APP_LABEL),
            'shop/admin/order-extra.html',
        ])

    def get_number(self, obj):
        return obj.get_number()
    get_number.short_description = pgettext_lazy('admin', "Order number")

    def get_total(self, obj):
        return str(obj.total)
    get_total.short_description = pgettext_lazy('admin', "Total")

    def get_subtotal(self, obj):
        return str(obj.subtotal)
    get_subtotal.short_description = pgettext_lazy('admin', "Subtotal")

    def get_outstanding_amount(self, obj):
        return str(obj.outstanding_amount)
    get_outstanding_amount.short_description = pgettext_lazy('admin', "Outstanding amount")

    def is_fully_paid(self, obj):
        return obj.is_fully_paid()
    is_fully_paid.short_description = pgettext_lazy('admin', "Is fully paid")
    is_fully_paid.boolean = True

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def render_as_html_extra(self, obj):
        return self.extra_template.render(obj.extra)
    render_as_html_extra.short_description = pgettext_lazy('admin', "Extra data")

    def get_customer_link(self, obj):
        try:
            url = reverse('admin:shop_customerproxy_change', args=(obj.customer.pk,))
            return format_html('<a href="{0}" target="_new">{1}</a>', url, obj.customer.get_username())
        except NoReverseMatch:
            return format_html('<strong>{0}</strong>', obj.customer.get_username())
    get_customer_link.short_description = pgettext_lazy('admin', "Customer")

    def get_search_fields(self, request):
        search_fields = list(super(BaseOrderAdmin, self).get_search_fields(request))
        search_fields.extend(['customer__user__email', 'customer__user__last_name'])
        try:
            # if CustomerModel contains a number field, let search for it
            if isinstance(CustomerModel._meta.get_field('number'), Field):
                search_fields.append('customer__number')
        except FieldDoesNotExist:
            pass
        return search_fields

    def get_form(self, request, obj=None, **kwargs):
        ModelForm = super(BaseOrderAdmin, self).get_form(request, obj, **kwargs)
        if obj:
            # store the requested transition inside the instance, so that the model's `clean()` method can access it
            obj._fsm_requested_transition = self._get_requested_transition(request)
        return ModelForm

    def change_view(self, request, object_id, form_url='', extra_context=None):
        assert object_id, "An Order object can not be added through the Django-Admin"
        current_status = self.get_object(request, object_id).status
        response = super(BaseOrderAdmin, self).change_view(request, object_id, form_url, extra_context)
        order = self.get_object(request, object_id)
        if current_status != order.status:
            transition_change_notification(order)
        return response


class PrintInvoiceAdminMixin(object):
    """
    A customized OrderAdmin class shall inherit from this mixin class, to add
    methods for printing the the invoice.
    """
    def get_fields(self, request, obj=None):
        fields = list(super(PrintInvoiceAdminMixin, self).get_fields(request, obj))
        fields.append('print_out')
        return fields

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = list(super(PrintInvoiceAdminMixin, self).get_readonly_fields(request, obj))
        readonly_fields.append('print_out')
        return readonly_fields

    def get_urls(self):
        my_urls = [
            url(r'^(?P<pk>\d+)/print_invoice/$', self.admin_site.admin_view(self.render_invoice),
                name='print_invoice'),
        ]
        my_urls.extend(super(PrintInvoiceAdminMixin, self).get_urls())
        return my_urls

    def _render_content(self, request, pk, template):
        order = self.get_object(request, pk)
        context = {'request': request, 'render_label': 'print'}
        customer_serializer = app_settings.CUSTOMER_SERIALIZER(order.customer)
        order_serializer = OrderDetailSerializer(order, context=context)
        return template.render(context={
            'customer': customer_serializer.data,
            'order': order_serializer.data,
            'object': order,
        }, request=request)

    def render_invoice(self, request, pk=None):
        template = select_template([
            '{}/print/invoice.html'.format(app_settings.APP_LABEL.lower()),
            'shop/print/invoice.html'
        ])
        content = self._render_content(request, pk, template)
        return HttpResponse(content)

    def print_out(self, obj):
        if obj.status in ['ready_for_delivery']:
            link = reverse('admin:print_invoice', args=(obj.id,)), pgettext_lazy('admin', "Invoice")
            return format_html(
                '<span class="object-tools"><a href="{0}" class="viewsitelink" target="_new">{1}</a></span>',
                *link)
        return ''
    print_out.short_description = pgettext_lazy('admin', "Print out")



# -*- coding: utf-8 -*-
"""
A wrapper around Django's messages framework for easier integration with Javascript based messages.
"""
from __future__ import unicode_literals

import json
from django.contrib import messages as django_messages
from django.utils.encoding import force_text


def add_message(request, level, message, title=None, delay=None):
    if title is None:
        title = django_messages.DEFAULT_TAGS[level].capitalize()
    extra_tags = {'title': force_text(title), 'delay': delay}
    django_messages.add_message(request, level, force_text(message), extra_tags=json.dumps(extra_tags))


def success(request, message, title=None, delay=0):
    add_message(request, django_messages.SUCCESS, message, title, delay)


def warning(request, message, title=None, delay=0):
    add_message(request, django_messages.WARNING, message, title, delay)


def error(request, message, title=None, delay=0):
    add_message(request, django_messages.ERROR, message, title, delay)


def info(request, message, title=None, delay=0):
    add_message(request, django_messages.INFO, message, title, delay)


def debug(request, message, title=None, delay=0):
    add_message(request, django_messages.DEBUG, message, title, delay)


def get_messages_as_json(request):
    data = []
    for message in django_messages.get_messages(request):
        try:
            extra_tags = json.loads(message.extra_tags)
        except (TypeError, json.JSONDecodeError):
            extra_tags = {}
        heading = extra_tags.get('title', message.level_tag.capitalize())
        try:
            delay = int(float(extra_tags['delay']) * 1000)
        except (KeyError, ValueError):
            delay = None
        data.append({
            'level': message.level_tag,
            'heading': heading,
            'body': message.message,
            'delay': delay,
        })
    return data
